/*
 LSM303DLH: 3-axis Accelerometer / 3-axis Magnetometer Library for Arduino
   created by NeoCat <neocat@neocat.jp>
*/
#ifndef LSM303DLH_h
#define LSM303DLH_h
extern "C" {
#include <math.h>
#include <wiring.h>
}
#include <Wire.h>

// Circuit (Sparkfun breakout board)
// Analog 5 - SCL
// Analog 4 - SDA
// GND      - GND
// 5V       - Vin
// DRDY,SA,INT1,INT2 are not used (NC)

// magnetic sensor mesurement mode
#define ACCL_PM_SLEEP   0 // sleep mode
#define ACCL_PM_NORMAL  1 // normal mode
#define ACCL_PM_0_5HZ   2 // low-power mode, limit data rate to 0.5Hz
#define ACCL_PM_1HZ     3 // low-power mode, limit data rate to 1Hz
#define ACCL_PM_2HZ     4 // low-power mode, limit data rate to 2Hz
#define ACCL_PM_5HZ     5 // low-power mode, limit data rate to 5Hz
#define ACCL_PM_10HZ    6 // low-power mode, limit data rate to 10Hz

// accel sensor output data rate / LPF cut-off freq
#define ACCL_HZ_50    0  // LPF freq =  37Hz
#define ACCL_HZ_100   1  // LPF freq =  74Hz
#define ACCL_HZ_400   2  // LPF freq = 292Hz
#define ACCL_HZ_1000  3  // LPF freq = 780Hz


// magnetic sensor mesurement mode
#define MAG_PM_CONT    0   // continuous mesurement
#define MAG_PM_SINGLE  1   // get single value, then go to sleep mode
#define MAG_PM_SLEEP   3   // sleep mode

// magnetic sensor data output rate
#define MAG_HZ_0_75 0   // 0.75Hz
#define MAG_HZ_1_5  1   // 1.5Hz
#define MAG_HZ_3_0  2   // 3.0Hz
#define MAG_HZ_7_5  3   // 7.5Hz
#define MAG_HZ_15   4   // 15Hz
#define MAG_HZ_30   5   // 30Hz
#define MAG_HZ_75   6   // 75Hz


class LSM303DLH_Class {
    byte sa;
    void _write_reg(byte id, byte addr, byte val);
    byte _read_reg(byte id, byte addr);
    void get_value(byte id, byte addr, bool little_endian, short val[3]);
  public:
    void begin(byte accl_pm=ACCL_PM_10HZ, byte accl_hz=ACCL_HZ_50,
               byte mag_pm=MAG_PM_CONT, byte mag_hz=MAG_HZ_15, byte sa=0) {
      Wire.begin();
      this->sa = sa ? 1 : 0;
      // Turn on accel sensor
      write_reg(0x20, (accl_pm<<5)|(accl_hz<<3)|7); // CTRL_REG1
      // Set magnetic sensor data rate
      write_reg(0x00, mag_hz<<2); // CRA_REG_M
      // Turn on magnetic sensor
      write_reg(0x02, (mag_pm << 6)); // MR_REG_M
    }
    void stop();
    void get_accel(short val[3]);
    void get_magnetic(short val[3]);
    void write_reg(byte addr, byte val) {
      _write_reg(addr >= 0x20 ? 0x18|sa/*acc*/ : 0x1e/*mag*/, addr, val);
    }
    byte read_reg(byte addr) {
      return _read_reg(addr < 0x20 ? 0x18|sa/*acc*/ : 0x1e/*mag*/, addr);
    }
};

extern LSM303DLH_Class LSM303DLH;

#endif
